from datetime import timedelta, datetime
from typing import Union, Any
from timeit import default_timer as timer

DEFAULT_TIMESTAMP_10M_AGO = datetime = datetime.now() - timedelta(minutes=10)
DEFAULT_TIMESTAMP_10Y_AGO = datetime = datetime.now() - timedelta(days=365 * 10)


def parse_timestamp(candidate: Union[str, Any]) -> datetime:
    """
    Expected something like 2022-02-08T16:32:14.900292+0000
    :param candidate:
    :return:
    """
    if isinstance(candidate, str):
        try:
            iso_candidate = candidate.split('+', 1)[0]
            return datetime.fromisoformat(iso_candidate)
        except ValueError:
            raise ValueError(f"Invalid date passed: {candidate}")
    elif isinstance(candidate, datetime):
        return candidate
    else:
        raise ValueError(f"I don't know how to handle {candidate}")


def get_clock(start_time: float) -> str:
    seconds = timer() - start_time
    if seconds <= 60.0:
        return f"{seconds:.2f} secs"
    else:
        return f"{seconds / 60.0:.2f} min"



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Dict, Any

from suricatalog.time import DEFAULT_TIMESTAMP_10M_AGO, parse_timestamp


class BaseFilter(ABC):
    @abstractmethod
    def accept(self, data: Dict[Any, Any]) -> bool:
        raise NotImplementedError()


class AlwaysTrueFilter(BaseFilter):

    def accept(self, data: Dict[Any, Any]) -> bool:
        return True


class OnlyAlertsFilter(BaseFilter):

    def __init__(self):
        self.timestamp = DEFAULT_TIMESTAMP_10M_AGO

    def set_timestamp(self, timestamp: datetime):
        if not timestamp:
            raise ValueError("Missing timestamp")
        self.timestamp = timestamp

    def accept(self, data: Dict[Any, Any]) -> bool:
        try:
            event_timestamp = parse_timestamp(data['timestamp'])
            if event_timestamp <= self.timestamp:
                return False
            if 'event_type' in data and 'alert' == data['event_type']:
                return True
            return False
        except ValueError:
            return False


class NXDomainFilter(BaseFilter):

    def accept(self, data: Dict[Any, Any]) -> bool:
        """
        tail -f eve.json|jq -c 'select(.dns.rcode=="NXDOMAIN")'
        """
        if 'dns' in data and 'rcode' in data['dns'] and data['dns']['rcode'] == 'NXDOMAIN':
            return True
        return False


class WithPrintablePayloadFilter(BaseFilter):
    def accept(self, data: Dict[Any, Any]) -> bool:
        """
        cat ~/SuricataLog/test/eve.json | jq -r -c 'select(.event_type=="alert")|.payload'|base64 --decode
        :param data:
        :return:
        """
        if 'event_type' in data and 'alert' == data['event_type'] and 'payload_printable' in data and data['payload_printable']:
            return True
        return False


class TimestampFilter(BaseFilter):

    def __init__(self):
        self.timestamp = DEFAULT_TIMESTAMP_10M_AGO

    def set_timestamp(self, timestamp: datetime):
        if not timestamp:
            raise ValueError("Missing timestamp")
        self.timestamp = timestamp

    def accept(self, data: Dict[Any, Any]) -> bool:
        """
        Filter events on a given timestamp
        :param data:
        :return:
        """
        try:
            event_timestamp = parse_timestamp(data['timestamp'])
            if event_timestamp <= self.timestamp:
                return False
        except ValueError:
            return False
        return True



import json
import unittest
from unittest import IsolatedAsyncioTestCase, TestCase
from datetime import datetime
from pathlib import Path

from suricatalog.time import parse_timestamp
from suricatalog.filter import NXDomainFilter, WithPrintablePayloadFilter, TimestampFilter, AlwaysTrueFilter
from suricatalog.log import get_events_from_eve
from suricatalog.report import AggregatedFlowProtoReport, HostDataUseReport, TopUserAgents

BASEDIR = Path(__file__).parent
events = []


class FilterTestCase(TestCase):

    def test_with_printable_payload_filter(self):
        payload = json.loads('{"timestamp":"2022-02-08T16:32:14.900292+0000","flow_id":879564658970874,'
                             '"pcap_cnt":9146,"event_type":"alert","src_ip":"52.96.222.130","src_port":25,'
                             '"dest_ip":"10.2.8.102","dest_port":49880,"proto":"TCP","metadata":{"flowints":{'
                             '"applayer.anomaly.count":1}},"alert":{"action":"allowed","gid":1,'
                             '"signature_id":2260002,"rev":1,"signature":"SURICATA Applayer Detect protocol only one '
                             'direction","category":"Generic Protocol Command Decode","severity":3},"smtp":{"helo":"['
                             '173.166.146.112]"},"app_proto":"smtp","app_proto_tc":"failed",'
                             '"flow":{"pkts_toserver":3,"pkts_toclient":3,"bytes_toserver":198,"bytes_toclient":276,'
                             '"start":"2022-02-08T16:32:14.659706+0000"},"payload":"","payload_printable":"",'
                             '"stream":0,"packet":"AAgCHEevIOUqtpPxCABFAAAohfUAAIAGj5A0YN6CCgIIZgAZwth27Ik6+soFbVAQ'
                             '+vDMSAAA","packet_info":{"linktype":1},"host":"raspberrypi"}')
        data_filter = WithPrintablePayloadFilter()
        self.assertFalse(data_filter.accept(payload))
        payload = json.loads('{"timestamp":"2022-02-08T16:32:20.491791+0000","flow_id":879564658970874,'
                             '"pcap_cnt":9254,"event_type":"alert","src_ip":"52.96.222.130","src_port":25,'
                             '"dest_ip":"10.2.8.102","dest_port":49880,"proto":"TCP","metadata":{"flowints":{'
                             '"applayer.anomaly.count":1,"smtp.anomaly.count":1}},"tx_id":0,'
                             '"alert":{"action":"allowed","gid":1,"signature_id":2220000,"rev":1,'
                             '"signature":"SURICATA SMTP invalid reply","category":"Generic Protocol Command Decode",'
                             '"severity":3},"smtp":{"helo":"[173.166.146.112]"},"app_proto":"smtp",'
                             '"app_proto_tc":"failed","flow":{"pkts_toserver":9,"pkts_toclient":14,'
                             '"bytes_toserver":570,"bytes_toclient":1148,"start":"2022-02-08T16:32:14.659706+0000"},'
                             '"payload":"TV0NCg==","payload_printable":"220 PH0P220CA0030.outlook.office365.com",'
                             '"stream":1,"packet":"AAgCH","packet_info":{"linktype":1},"host":"raspberrypi"}')
        self.assertTrue(data_filter.accept(payload))

    def test_nxdomain(self):
        data_filter = NXDomainFilter()
        payload = json.loads('{"timestamp":"2022-02-23T19:04:05.606882+0000","flow_id":1452066252079368,'
                             '"pcap_cnt":85538,"event_type":"dns","src_ip":"172.16.0.170","src_port":58529,'
                             '"dest_ip":"172.16.0.52","dest_port":53,"proto":"UDP","dns":{"version":2,"type":"answer",'
                             '"id":46693,"flags":"8583","qr":true,"aa":true,"rd":true,"ra":true,'
                             '"rrname":"wpad.sunnystation.com","rrtype":"A","rcode":"NXDOMAIN","authorities":[{'
                             '"rrname":"sunnystation.com","rrtype":"SOA","ttl":3600,'
                             '"soa":{"mname":"sunnystation-dc.sunnystation.com","rname":"hostmaster.sunnystation.com",'
                             '"serial":32,"refresh":900,"retry":600,"expire":86400,"minimum":3600}}]},'
                             '"host":"raspberrypi"}')
        self.assertTrue(data_filter.accept(payload))
        payload = json.loads('{"timestamp":"2022-02-08T15:08:22.266264+0000","flow_id":127219202876190,'
                             '"pcap_cnt":4196,"event_type":"tls","src_ip":"10.2.8.102","src_port":49798,'
                             '"dest_ip":"185.248.140.40","dest_port":443,"proto":"TCP","tls":{"subject":"C=GB, '
                             'ST=London, L=London, O=Global Security, OU=IT Department, CN=example.com",'
                             '"issuerdn":"C=GB, ST=London, L=London, O=Global Security, OU=IT Department, '
                             'CN=example.com","serial":"0E:B4:42:BB:46:DA:06:75:CF:4B:CA:7B:2B:E8:C2:60:0B:18:32:17",'
                             '"fingerprint":"28:c1:5a:60:d5:cb:a4:89:4b:07:c6:4c:d8:67:e7:f1:76:ef:da:aa",'
                             '"version":"TLS 1.2","notbefore":"2022-01-31T20:50:00","notafter":"2023-01-31T20:50:00",'
                             '"ja3":{"hash":"51c64c77e60f3980eea90869b68c58a8","string":"771,'
                             '49196-49195-49200-49199-49188-49187-49192-49191-49162-49161-49172-49171-157-156-61-60'
                             '-53-47-10,10-11-13-35-23-65281,29-23-24,0"},"ja3s":{'
                             '"hash":"ec74a5c51106f0419184d0dd08fb05bc","string":"771,49200,65281-11-35-23"}},'
                             '"host":"raspberrypi"}')
        self.assertFalse(data_filter.accept(payload))


class ReportTestCase(IsolatedAsyncioTestCase):
    eve_list = []
    netflow_eve_list = []

    @classmethod
    def setUpClass(cls) -> None:
        with open(BASEDIR.joinpath("eve.json"), 'rt') as eve_file:
            for event in eve_file:
                ReportTestCase.eve_list.append(json.loads(event))
        with open(BASEDIR.joinpath("eve_udp_flow.json"), 'rt') as eve_file:
            for event in eve_file:
                ReportTestCase.netflow_eve_list.append(json.loads(event))

    async def test_aggregated_flow_proto_report(self):
        events.append("test_aggregated_flow_proto_report")
        can_rep = AggregatedFlowProtoReport()
        for alert in ReportTestCase.eve_list:
            await can_rep.ingest_data(alert)
        self.assertIsNotNone(can_rep.port_proto_count)
        self.assertEqual(119, len(can_rep.port_proto_count.keys()))
        self.assertEqual(336, can_rep.port_proto_count[('TCP', 443)])

    async def test_host_data_use_report(self):
        events.append("test_host_data_use_report")
        hr = HostDataUseReport()
        for netflow_event in ReportTestCase.netflow_eve_list:
            await hr.ingest_data(netflow_event, '224.0.0.251')
        self.assertEqual(153339, hr.bytes)
        hr2 = HostDataUseReport()
        for netflow_event in ReportTestCase.netflow_eve_list:
            await hr2.ingest_data(netflow_event, '127.0.0.1')
            await hr.ingest_data(netflow_event, '127.0.0.1')
        self.assertEqual(0, hr2.bytes)
        self.assertEqual(153339, hr.bytes)

    async def test_top_user_agents(self):
        events.append("test_top_user_agents")
        tua = TopUserAgents()
        for event in ReportTestCase.eve_list:
            await tua.ingest_data(event)

        self.assertIsNotNone(tua.agents)
        self.assertTrue('test' in tua.agents)
        self.assertEqual(10, len(tua.agents))


class SuricataLogTestCase(unittest.TestCase):
    eve_list = []
    old_date = datetime.fromisoformat('2021-02-08T16:32:14.900292')

    @classmethod
    def setUpClass(cls) -> None:
        with open(BASEDIR.joinpath("eve.json"), 'rt') as eve_file:
            for event in eve_file:
                SuricataLogTestCase.eve_list.append(json.loads(event))

    def test_parse_timestamp(self):
        self.assertTrue(parse_timestamp('2022-02-08T16:32:14.900292+0000'))
        self.assertTrue(parse_timestamp('2022-02-08 16:32:14.900292+0000'))
        self.assertTrue(parse_timestamp('2022-02-08T16:32:14.900292'))
        try:
            parse_timestamp('XXX-02-08T16:32:14.900292')
            self.fail("Was supposed to fail with an invalid timestamp")
        except ValueError:
            pass

    def test_timestamp_filter(self):
        timestamp_filter = TimestampFilter()
        timestamp_filter.timestamp = SuricataLogTestCase.old_date
        self.assertTrue(
            timestamp_filter.accept(data=SuricataLogTestCase.eve_list[0])
        )
        self.assertTrue(
            timestamp_filter.accept(data=SuricataLogTestCase.eve_list[132])
        )

    def test_get_alerts(self):
        timestamp_filter = TimestampFilter()
        timestamp_filter.timestamp = SuricataLogTestCase.old_date
        all_alerts = [x for x in get_events_from_eve(
            eve_files=[BASEDIR.joinpath("eve.json")],
            data_filter=timestamp_filter
        ) if x['event_type'] == 'alert']
        self.assertIsNotNone(all_alerts)
        self.assertEqual(275, len(all_alerts))
        self.assertEqual('SURICATA Applayer Detect protocol only one direction', all_alerts[90]['alert']['signature'])

    def test_get_all_events(self):
        always_true_filter = AlwaysTrueFilter()
        all_events = [x for x in get_events_from_eve(
            eve_files=[BASEDIR.joinpath("eve.json")],
            data_filter=always_true_filter
        )]
        self.assertIsNotNone(all_events)
        self.assertListEqual(SuricataLogTestCase.eve_list, all_events)


if __name__ == '__main__':
    unittest.main()



import json
import os
import time
from json import JSONDecodeError
from pathlib import Path
from typing import Callable, Dict

from suricatalog.filter import BaseFilter

DEFAULT_EVE = [Path("/var/log/suricata/eve.json")]


def get_events_from_eve(
        *,
        eve_files=None,
        data_filter: BaseFilter,
) -> Dict:
    """
    Get alerts from a JSON even file. Assumed each line is a valid
    JSON document, otherwise reader will crash
    :param eve_files:
    :param data_filter: Filter events based on several criteria
    :return:
    """
    if eve_files is None:
        eve_files = DEFAULT_EVE
    for eve_file in eve_files:
        with open(eve_file, 'rt') as eve:
            for line in eve:
                try:
                    data = json.loads(line)
                    if data_filter.accept(data):
                        yield data
                except JSONDecodeError:
                    continue  # Try to read the next record


def tail_eve(
        *,
        eve_file=None,
        decorator: Callable = json.loads
):
    """
    Similar to UNIX ``tail -f eve.json``
    :param decorator:
    :param eve_file:
    :return:
    """
    if eve_file is None:
        eve_file = DEFAULT_EVE
    with open(eve_file, 'rt') as eve:
        eve.seek(0, os.SEEK_END)
        while True:
            try:
                line = eve.readline()
                if not line:
                    time.sleep(0.1)
                    continue
                data = decorator(line)
                yield data
            except JSONDecodeError:
                continue



import locale
import textwrap
from pathlib import Path
from typing import Type, List, Any, Dict, Union

from textual import on, work
from textual.app import App, ComposeResult, CSSPathType
from textual.driver import Driver
from textual.screen import ModalScreen
from textual.widgets import Footer, Header, Pretty, DataTable, Button

from suricatalog.log import get_events_from_eve
from suricatalog.filter import BaseFilter

locale.setlocale(locale.LC_ALL, '')
BASEDIR = Path(__file__).parent


class DetailScreen(ModalScreen):
    ENABLE_COMMAND_PALETTE = False
    CSS_PATH = BASEDIR.joinpath('css').joinpath('details_screen.tcss')

    def __init__(
            self,
            name: str | None = None,
            ident: str | None = None,
            classes: str | None = None,
            data: str = None
    ):
        super().__init__(name, ident, classes)
        self.data = data

    def compose(self) -> ComposeResult:
        yield Pretty(self.data)
        button = Button("Close", variant="primary", id="close")
        button.tooltip = "Go back to main screen"
        yield button

    @on(Button.Pressed, "#close")
    def on_button_pressed(self, _) -> None:
        self.app.pop_screen()


class BaseAlert(App):

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
    ):
        super().__init__(driver_class, css_path, watch_css)
        self.eve_files = None
        self.filter = None

    @staticmethod
    def __get_key_from_map__(map1: Dict[str, Any], keys: List[str]):
        """
        Return the first matching key from a map
        :param map1:
        :param keys:
        :return: Nothing ig none of the keys are in the map
        """
        val = ""
        for key in keys:
            if key in map1:
                val = map1[key]
                break
        return val

    @staticmethod
    async def __extract__from_alert__(alert: Dict[str, Any]) -> Dict[str, Any]:
        timestamp = alert['timestamp']
        dest_port = str(BaseAlert.__get_key_from_map__(alert, ['dest_port']))
        dest_ip = BaseAlert.__get_key_from_map__(alert, ['dest_ip'])
        src_ip = BaseAlert.__get_key_from_map__(alert, ['src_ip'])
        src_port = str(BaseAlert.__get_key_from_map__(alert, ['src_port']))
        protocol = BaseAlert.__get_key_from_map__(alert, ['app_proto', 'proto'])
        severity = alert['alert']['severity']
        signature = alert['alert']['signature'],
        payload_printable = alert['payload_printable'] if 'payload_printable' in alert else ""
        return {
            "timestamp": timestamp,
            "dest_port": dest_port,
            "src_ip": src_ip,
            "dest_ip": dest_ip,
            "src_port": src_port,
            "protocol": protocol,
            "severity": severity,
            "signature": signature,
            "payload_printable": payload_printable
        }

    def set_filter(self, the_filter: BaseFilter):
        if not the_filter:
            raise ValueError("Filter is required")
        self.filter = the_filter

    def set_eve_files(self, eve_files: List[Path]):
        if not eve_files:
            raise ValueError("One or more eve files is required")
        self.eve_files = eve_files


class TableAlert(BaseAlert):
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
    ):
        super().__init__(driver_class, css_path, watch_css)
        self.events: Union[Dict[Dict[str, any]]] = {}

    def compose(self) -> ComposeResult:
        yield Header()
        alerts_tbl = DataTable()
        alerts_tbl.show_header = True
        alerts_tbl.add_column("Timestamp")
        alerts_tbl.add_column("Severity")
        alerts_tbl.add_column("Signature")
        alerts_tbl.add_column("Protocol")
        alerts_tbl.add_column("Destination")
        alerts_tbl.add_column("Source")
        alerts_tbl.add_column("Payload")
        alerts_tbl.zebra_stripes = True
        alerts_tbl.loading = True
        alerts_tbl.cursor_type = 'row'
        alerts_tbl.tooltip = textwrap.dedent("""
        Suricata alert details. Select a row to get full details.
        """)
        yield alerts_tbl
        yield Footer()

    @work(exclusive=False)
    async def on_mount(self) -> None:
        alerts_tbl = self.query_one(DataTable)
        alert_cnt = 0
        alerts_tbl.loading = False
        for event in get_events_from_eve(
                data_filter=self.filter,
                eve_files=self.eve_files
        ):
            if not self.filter.accept(event):
                continue
            brief_data = await BaseAlert.__extract__from_alert__(event)
            timestamp = brief_data['timestamp']
            alerts_tbl.add_row(
                timestamp,
                brief_data['severity'],
                brief_data['signature'],
                brief_data['protocol'],
                f"{brief_data['dest_ip']}:{brief_data['dest_port']}",
                f"{brief_data['src_ip']}:{brief_data['src_port']}",
                brief_data['payload_printable']
            )
            alert_cnt += 1
            self.events[timestamp] = event
        alerts_tbl.sub_title = f"Total alerts: {alert_cnt}"

    @on(DataTable.HeaderSelected)
    def on_header_clicked(self, event: DataTable.HeaderSelected):
        alerts_tbl: DataTable = event.data_table
        if event.column_index < 2:
            alerts_tbl.sort(event.column_key)

    @on(DataTable.RowSelected)
    def on_row_clicked(self, event: DataTable.RowSelected) -> None:
        table = event.data_table
        row_key = event.row_key
        row = table.get_row(row_key)
        data = self.events[row[0]]
        runner_detail = DetailScreen(data=data)
        self.push_screen(runner_detail)

    def action_quit_app(self) -> None:
        self.exit("Exiting Alerts now...")



import textwrap
from pathlib import Path
from typing import Type, List

from textual import on, work
from textual.app import App, ComposeResult, CSSPathType
from textual.driver import Driver
from textual.widgets import Header, DataTable, Footer, Digits, RichLog

from suricatalog.filter import BaseFilter
from suricatalog.log import get_events_from_eve
from suricatalog.report import AggregatedFlowProtoReport, HostDataUseReport, TopUserAgents

BASEDIR = Path(__file__).parent


class FlowApp(App):
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            data_filter: BaseFilter = None,
            eve: List[Path] = None
    ):
        super().__init__(driver_class, css_path, watch_css)
        self.data_filter = data_filter
        self.eve = eve

    def action_quit_app(self) -> None:
        self.exit("Exiting Net-Flow now...")

    def compose(self) -> ComposeResult:
        yield Header()
        alerts_tbl = DataTable()
        alerts_tbl.show_header = True
        alerts_tbl.add_column("Destination IP")
        alerts_tbl.add_column("Port")
        alerts_tbl.add_column("Count")
        alerts_tbl.zebra_stripes = True
        alerts_tbl.loading = True
        alerts_tbl.cursor_type = 'row'
        alerts_tbl.tooltip = textwrap.dedent("""
        Network flow details
        """)
        yield alerts_tbl
        yield Footer()

    async def on_mount(self) -> None:
        alerts_tbl = self.query_one(DataTable)
        alert_cnt = 0
        afr = AggregatedFlowProtoReport()
        for event in get_events_from_eve(
                eve_files=self.eve,
                data_filter=self.data_filter):
            if not self.data_filter.accept(event):
                continue
            await afr.ingest_data(event)
        alerts_tbl.loading = False
        for (dest_ip_port, cnt) in afr.port_proto_count.items():
            alerts_tbl.add_row(
                dest_ip_port[0],
                str(dest_ip_port[1]),
                str(cnt)
            )
            alert_cnt += 1

    @on(DataTable.HeaderSelected)
    def on_header_clicked(self, event: DataTable.HeaderSelected):
        alerts_tbl: DataTable = event.data_table
        alerts_tbl.sort(event.column_key)


class HostDataUse(App):
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    CSS_PATH = BASEDIR.joinpath('css').joinpath('canned.tcss')
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            ip_address: str = None,
            data_filter: BaseFilter = None,
            eve: List[Path] = None

    ):
        super().__init__(driver_class, css_path, watch_css)
        self.ip_address = ip_address
        self.data_filter: BaseFilter = data_filter
        self.eve = eve

    def action_quit_app(self) -> None:
        self.exit("Exiting Net-Flow now...")

    def compose(self) -> ComposeResult:
        yield Header()
        digits = Digits(id="netflow")
        digits.loading = True
        digits.tooltip = textwrap.dedent("""
            Net FLow in bytes.
            """)
        yield digits
        yield Footer()

    @work(exclusive=False)
    async def on_mount(self) -> None:
        host_data_user_report = HostDataUseReport()
        for event in get_events_from_eve(
                eve_files=self.eve,
                data_filter=self.data_filter):
            await host_data_user_report.ingest_data(event, self.ip_address)
        digits = self.query_one('#netflow', Digits)
        digits.update(f"{host_data_user_report.bytes:n} bytes")
        digits.loading = False


class TopUserApp(App):
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    CSS_PATH = BASEDIR.joinpath('css').joinpath('canned.tcss')
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            data_filter: BaseFilter = None,
            eve: List[Path] = None
    ):
        super().__init__(driver_class, css_path, watch_css)
        self.data_filter = data_filter
        self.eve_files = eve

    def action_quit_app(self) -> None:
        self.exit("Exiting Top user now...")

    def compose(self) -> ComposeResult:
        yield Header()
        pretty = RichLog(id="agent")
        pretty.loading = True
        yield pretty
        yield Footer()

    @work(exclusive=False)
    async def on_mount(self) -> None:
        top_user_agents = TopUserAgents()
        log = self.query_one("#agent", RichLog)
        log.loading = False
        for event in get_events_from_eve(
                eve_files=self.eve_files,
                data_filter=self.data_filter):
            await top_user_agents.ingest_data(event)
        log.write(top_user_agents.agents)


class OneShotApp(App):
    BINDINGS = [
        ("q", "quit_app", "Quit")
    ]
    ENABLE_COMMAND_PALETTE = False

    def __init__(
            self,
            driver_class: Type[Driver] | None = None,
            css_path: CSSPathType | None = None,
            watch_css: bool = False,
            eve: List[Path] = None,
            data_filter: BaseFilter = None
    ):
        super().__init__(driver_class, css_path, watch_css)
        self.data_filter = data_filter
        self.eve = eve

    def action_quit_app(self) -> None:
        self.exit(f"Exiting {self.title} now...")

    def compose(self) -> ComposeResult:
        yield Header()
        log = RichLog(id='events')
        log.loading = True
        yield log
        yield Footer()

    async def pump_events(self, log: RichLog):
        loaded = 0
        for single_alert in get_events_from_eve(eve_files=self.eve, data_filter=self.data_filter):
            log.loading = False
            log.write(single_alert)
            loaded += 1

    @work(exclusive=False)
    async def on_mount(self):
        log = self.query_one('#events', RichLog)
        await self.pump_events(log)
